/*****************************************************************************
*   Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.         *
*   Site   : http://space.tin.it/computer/giparrel                           *
*   E-Mail : joparrel@tin.it, giusparr@tin.it, giparre@tin.it                *
*                                                                            *
*   Revision:   2.1  (July 31, 1998).                                        *
*                                                                            *
*   Definition of JPDialog class.                                            *
*                                                                            *
*   Read JPDIALOG.TXT for use and license.                                   *
*                                                                            *
*   Only for OWL 5.02 and 5.01 users.                                        *
*   OWL and ObjectWindows are trademarks of Borland International.           *
*                                                                            *
*   THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF       *
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO      *
*   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A           *
*   PARTICULAR PURPOSE.                                                      *
*****************************************************************************/
#include <owl/pch.h>

#include "jpdialog.h"


//{{JPDialog Implementation}}

JPDialog::JPDialog(TWindow* parent,
                   TResId resId,
                   TModule* module,
                   JPTooltipSupport* mytooltip,
                   bool FullTooltipSupport,
                   bool AlwaysForceRectAssociate,
                   bool ForceRectAssociateIfDisabled,
                   bool EnableStatusbarMessages)
:
    TDialog(parent, resId, module)
{
  GlobalTooltip = mytooltip;
  isFullSupport = FullTooltipSupport;
  forceRectAssociate = AlwaysForceRectAssociate;
  forceDisabledAssociate = ForceRectAssociateIfDisabled;
  currentHwnd = NULL;
  decFrame = NULL;
  sbarMessage = EnableStatusbarMessages;
}


JPDialog::~JPDialog()
{
  HWND hwnd = GetHandle();
  if ((hwnd) && (GlobalTooltip))
  	  GlobalTooltip->AssociateTooltips(hwnd, false, isFullSupport, forceRectAssociate, forceDisabledAssociate);
  SetStatusbarMessage(-1);
  Destroy(IDCANCEL);
}

bool JPDialog::EvInitDialog(HWND hWndFocus)
{
  bool result;

  result = TDialog::EvInitDialog(hWndFocus);

  // INSERT>> Your code here.
  HWND hwnd = GetHandle();
  if ((hwnd) && (GlobalTooltip))
  	  GlobalTooltip->AssociateTooltips(hwnd, true, isFullSupport, forceRectAssociate, forceDisabledAssociate);

  if ((sbarMessage) && (GlobalTooltip))
     {
     TWindow* parentWin = GetParentO();
     decFrame = parentWin ? TYPESAFE_DOWNCAST(parentWin, TDecoratedFrame) : 0;
     while (parentWin && !decFrame)
          {
          parentWin = parentWin->GetParentO();
          if (parentWin)
             decFrame = TYPESAFE_DOWNCAST(parentWin, TDecoratedFrame);
          }
     }
  return result;
}

void
JPDialog::SetStatusbarMessage(int Id)
{
   if (decFrame)
      {
      if (Id > 0)
         decFrame->HandleMessage(WM_MENUSELECT, Id, 0);
      else
         {
         #if defined(BI_PLAT_WIN32)
         decFrame->HandleMessage(WM_MENUSELECT, (TParam1) 0xFFFF0000, (TParam2) 0);
         #else
         decFrame->HandleMessage(WM_MENUSELECT, (TParam1) 0, (TParam2) 0x0000FFFFL);
         #endif
         }
      decFrame->HandleMessage(WM_ENTERIDLE, MSGF_MENU);
      }
}

HWND
JPDialog::FindChildFromPoint(HWND hwnd, POINT Point)
{
  HWND hwndValue = NULL;
  HWND child = ::GetWindow(hwnd, GW_CHILD);
  while (::IsWindow(child))
        {
        TRect rect;
        ::GetWindowRect(child, &rect);
        int Id = ::GetDlgCtrlID(child);
        if (rect.Contains(Point) && (Id > 0) && (Id != (int) 0xFFFF))
           {
           hwndValue = child;
           break;
           }
        child = ::GetWindow(child, GW_HWNDNEXT);
        }
  return hwndValue;
}

bool JPDialog::IdleAction(long idleCount)
{
   if ((idleCount == 0) && decFrame)
      {
      TPoint curLoc;
      GetCursorPos(curLoc);
      HWND hwnd = FindChildFromPoint(*this, curLoc);
      if ((hwnd) && (hwnd != currentHwnd))
         {
         int Id = ::GetDlgCtrlID(hwnd);
         if ((Id > 0) && (Id != (int) 0xFFFF))
            {
            currentHwnd = hwnd;
            SetStatusbarMessage(Id);
            }
         }
      else if ((hwnd == 0) || (hwnd == GetHandle()))
              {
              currentHwnd = hwnd;
              SetStatusbarMessage(-1);
              }
      }
   return TDialog::IdleAction(idleCount);
}

