/*******************************************************************************
*  Copyright (c) 1997, 1998 by Joseph Parrello. All Rights Reserved.           *
*  Site   : http://space.tin.it/computer/giparrel                              *
*  E-Mail : joparrel@tin.it, giusparr@tin.it, giparre@tin.it                   *
*                                                                              *
*  Revision:   2.0  (July 31, 1998).                                           *
*                                                                              *
*  Definition of JPPropertySheet and JPPropertyPage classes.                   *
*                                                                              *
*  Read JPPRPAGE.TXT for use and license.                                      *
*                                                                              *
*  Only for OWL 5.02 and 5.01 users.                                           *
*  OWL and ObjectWindows are trademarks of Borland International.              *
*                                                                              *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF          *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO         *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A              *
*  PARTICULAR PURPOSE.                                                         *
*******************************************************************************/
#include <owl/pch.h>

#include "jpprpage.h"

//{{JPPropertyPage Implementation}}


//////////////////////////////////////////////////////////////////
//               JPPropertySheet class                          //
//////////////////////////////////////////////////////////////////

IMPLEMENT_CASTABLE1(JPPropertySheet, TPropertySheet);

DEFINE_RESPONSE_TABLE1(JPPropertySheet, TPropertySheet)
  EV_WM_SETCURSOR,
END_RESPONSE_TABLE;

JPPropertySheet::JPPropertySheet(TWindow* parent,
                                 const char far* title,
                                 uint startPage,
                                 bool isWizard,
                                 uint32 flags,
                                 TModule* module,
                                 JPTooltipSupport* mytooltip,
                                 bool FullTooltipSupport,
                                 bool AlwaysForceRectAssociate,
                                 bool ForceRectAssociateIfDisabled,
                                 bool EnableStatusbarMessages)
:
    TPropertySheet(parent, title, startPage, isWizard, flags, module)
{
  // INSERT>> Your constructor code here.
  forceRectAssociate = AlwaysForceRectAssociate;
  forceDisabledAssociate = ForceRectAssociateIfDisabled;
  GlobalSheetTooltip = mytooltip;
  isFullSupport = FullTooltipSupport;
  currentHwnd = NULL;
  decFrame = NULL;
  sbarMessage = EnableStatusbarMessages;
  if ((sbarMessage) && (GlobalSheetTooltip))
     {
     TWindow* parentWin = GetParentO();
     decFrame = parentWin ? TYPESAFE_DOWNCAST(parentWin, TDecoratedFrame) : 0;
     while (parentWin && !decFrame)
          {
          parentWin = parentWin->GetParentO();
          if (parentWin)
             decFrame = TYPESAFE_DOWNCAST(parentWin, TDecoratedFrame);
          }
     if (sbarMessage && decFrame)
        EnableSubclass(true);
     }
}


JPPropertySheet::~JPPropertySheet()
{
  Destroy(IDCANCEL);

  // INSERT>> Your destructor code here.
}

void JPPropertySheet::SetupWindow()
{
  TPropertySheet::SetupWindow();

  // INSERT>> Your code here.
  HWND hwnd = GetHandle();
  if ((hwnd) && (GlobalSheetTooltip))
     GlobalSheetTooltip->AssociateTooltips(hwnd, true, isFullSupport, forceRectAssociate, forceDisabledAssociate);
}


void JPPropertySheet::CleanupWindow()
{
  TPropertySheet::CleanupWindow();

  // INSERT>> Your code here.
  HWND hwnd = GetHandle();
  if ((hwnd) && (GlobalSheetTooltip))
     GlobalSheetTooltip->AssociateTooltips(hwnd, false, isFullSupport, forceRectAssociate, forceDisabledAssociate);
  SetStatusbarMessage(-1);
}

JPTooltipSupport* JPPropertySheet::GetGlobalTooltip()
{
  return GlobalSheetTooltip;
}

void
JPPropertySheet::SetStatusbarMessage(int Id)
{
   if (decFrame)
      {
      if (Id > 0)
         decFrame->HandleMessage(WM_MENUSELECT, Id, 0);
      else
         {
         #if defined(BI_PLAT_WIN32)
         decFrame->HandleMessage(WM_MENUSELECT, (TParam1) 0xFFFF0000, (TParam2) 0);
         #else
         decFrame->HandleMessage(WM_MENUSELECT, (TParam1) 0, (TParam2) 0x0000FFFFL);
         #endif
         }
      decFrame->HandleMessage(WM_ENTERIDLE, MSGF_MENU);
      }
}

HWND JPPropertySheet::FindChildFromPoint(HWND hwnd, POINT Point)
{
  HWND hwndValue = NULL;
  HWND child = ::GetWindow(hwnd, GW_CHILD);
  while (::IsWindow(child))
        {
        TRect rect;
        ::GetWindowRect(child, &rect);
        int Id = ::GetDlgCtrlID(child);
        if (rect.Contains(Point) && (Id > 0) && (Id != (int) 0xFFFF))
           {
           hwndValue = child;
           break;
           }
        child = ::GetWindow(child, GW_HWNDNEXT);
        }
  return hwndValue;
}

bool JPPropertySheet::EvSetCursor(HWND hWndCursor, uint hitTest, uint mouseMsg)
{
   if ((decFrame) && (mouseMsg == WM_MOUSEMOVE) && (hitTest == HTCLIENT))
      {
      TPoint curLoc;
      GetCursorPos(curLoc);
      HWND hwnd = FindChildFromPoint(*this, curLoc);
      if ((hwnd) && (hwnd != currentHwnd))
         {
         int Id = ::GetDlgCtrlID(hwnd);
         if ((Id > 0) && (Id != (int) 0xFFFF))
            {
            currentHwnd = hwnd;
            SetStatusbarMessage(Id);
            }
         }
      else if ((hwnd == 0) || (hwnd == GetHandle()))
              {
              currentHwnd = hwnd;
              SetStatusbarMessage(-1);
              }
      }
   return TPropertySheet::EvSetCursor(hWndCursor, hitTest, mouseMsg);
}

TDecoratedFrame* JPPropertySheet::GetDecoratedFrame()
{
    return decFrame;
}

//////////////////////////////////////////////////////////////////
//               JPPropertyPage class                           //
//////////////////////////////////////////////////////////////////

IMPLEMENT_CASTABLE1(JPPropertyPage, TPropertyPage);

DEFINE_RESPONSE_TABLE1(JPPropertyPage, TPropertyPage)
  EV_WM_SETCURSOR,
END_RESPONSE_TABLE;

JPPropertyPage::JPPropertyPage(JPPropertySheet* parent,
                               TResId resId,
                               const char far* title,
                               TResId iconRes,
                               TModule* module,
                               bool FullTooltipSupport,
                               bool AlwaysForceRectAssociate,
                               bool ForceRectAssociateIfDisabled,
                               bool EnableStatusbarMessages)
:
    TPropertyPage(parent, resId, title, iconRes, module)
{
  psheet = parent;
  forceRectAssociate = AlwaysForceRectAssociate;
  forceDisabledAssociate = ForceRectAssociateIfDisabled;
  isFullSupport = FullTooltipSupport;
  currentHwnd = NULL;
  decFrame = NULL;
  sbarMessage = EnableStatusbarMessages;

  // Retrieve the global tooltip from parent window.
  GlobalTooltip = parent->GetGlobalTooltip();

  // If parent window has the style PSH_HASHELP set, then set this flag in this page too.
  PROPSHEETHEADER psh;
  psh = parent->GetPropSheetHeader();
  if (psh.dwFlags & PSH_HASHELP)
     PageInfo.dwFlags |= PSP_HASHELP;
}


JPPropertyPage::~JPPropertyPage()
{
  Destroy(IDCANCEL);
}

void JPPropertyPage::SetupWindow()
{
  TPropertyPage::SetupWindow();

  // INSERT>> Your code here.
  HWND hwnd = GetHandle();
  if ((hwnd) && (GlobalTooltip))
     GlobalTooltip->AssociateTooltips(hwnd, true, isFullSupport, forceRectAssociate, forceDisabledAssociate);

  if (sbarMessage)
     decFrame = psheet->GetDecoratedFrame();
}

void JPPropertyPage::CleanupWindow()
{
  TPropertyPage::CleanupWindow();

  // INSERT>> Your code here.
  HWND hwnd = GetHandle();
  if ((hwnd) && (GlobalTooltip))
     GlobalTooltip->AssociateTooltips(hwnd, false, isFullSupport, forceRectAssociate, forceDisabledAssociate);
}

void JPPropertyPage::SetGlobalTooltip(JPTooltipSupport* mytooltip)
{
  GlobalTooltip = mytooltip;
}

bool JPPropertyPage::EvSetCursor(HWND hWndCursor, uint hitTest, uint mouseMsg)
{
   if ((decFrame) && (mouseMsg == WM_MOUSEMOVE) && (hitTest == HTCLIENT))
      {
      TPoint curLoc;
      GetCursorPos(curLoc);
      HWND hwnd = psheet->FindChildFromPoint(*this, curLoc);
      if ((hwnd) && (hwnd != currentHwnd))
         {
         int Id = ::GetDlgCtrlID(hwnd);
         if ((Id > 0) && (Id != (int) 0xFFFF))
            {
            currentHwnd = hwnd;
            psheet->SetStatusbarMessage(Id);
            }
         }
      else if ((hwnd == 0) || (hwnd == GetHandle()))
              {
              currentHwnd = hwnd;
              psheet->SetStatusbarMessage(-1);
              }
      }
   return TPropertyPage::EvSetCursor(hWndCursor, hitTest, mouseMsg);
}

