/*******************************************************************************
*  Copyright (c) 1997, 1998 by Joseph Parrello. All Rights Reserved.           *
*  Site   : http://space.tin.it/computer/giparrel                              *
*  E-Mail : joparrel@tin.it, giusparr@tin.it, giparre@tin.it                   *
*                                                                              *
*  Revision:   2.0  (July 31, 1998).                                           *
*                                                                              *
*  Definition of JPTooltipSupport class.                                       *
*                                                                              *
*  Read JPTOOLTP.TXT for use and license.                                      *
*                                                                              *
*  Only for OWL 5.02 and 5.01 users.                                           *
*  OWL and ObjectWindows are trademarks of Borland International.              *
*                                                                              *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF          *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO         *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A              *
*  PARTICULAR PURPOSE.                                                         *
*******************************************************************************/
#include <owl/pch.h>

#include "jptooltp.h"


//{{JPTooltipSupport Implementation}}
#if defined(BI_PLAT_WIN16)
FARPROC  lpfnFilterProc=NULL;   // Needed by MakeProcInstance only for Win16 targets
#endif
HHOOK    lpfnOldHook=NULL;
static TTooltip*  HookProcTooltip;

#pragma warn -sig
#if defined(BI_PLAT_WIN16)
int far pascal __export
#elif defined(BI_PLAT_WIN32)
__stdcall
#endif
HookFilterFunc(int nCode, WPARAM wParam, LPARAM lParam)
{
   if (nCode == HC_ACTION)
      {
      MSG FAR * lpmsg = (MSG FAR *) lParam;
      switch (lpmsg->message) {
           case WM_MOUSEMOVE:
           case WM_LBUTTONDOWN:
           case WM_LBUTTONUP:
           case WM_MBUTTONDOWN:
           case WM_MBUTTONUP:
           case WM_RBUTTONDOWN:
           case WM_RBUTTONUP:
               {
                MSG msg;
                msg.lParam = lpmsg->lParam;
                msg.wParam = lpmsg->wParam;
                msg.message = lpmsg->message;
                msg.hwnd = (HWND) lpmsg->hwnd;
                if (HookProcTooltip && HookProcTooltip->GetHandle())
                   HookProcTooltip->RelayEvent((MSG&) msg);
               }
               break;
           default:
               break;
           }
      }
   return ::CallNextHookEx(lpfnOldHook, nCode, wParam, lParam);
}
#pragma warn .sig

JPTooltipSupport::JPTooltipSupport(TWindow* parent)
{
  // INSERT>> Your constructor code here.
  lpfnOldHook = NULL;
  HookProcTooltip = NULL;
  if (parent)
   {
     parentwindow = parent;
     HookProcTooltip = new TTooltip(parent, true);
     HookProcTooltip->Create();
     HookProcTooltip->Activate(true);
     EnableHookProc(true);
   }
}


JPTooltipSupport::~JPTooltipSupport()
{
  TToolInfo ti;

  EnableHookProc(false);
  if (HookProcTooltip)
     {
     while (HookProcTooltip->GetToolCount()>0)
        {
        if (HookProcTooltip->EnumTools(0, ti))
            HookProcTooltip->DeleteTool(ti);
        }
     delete HookProcTooltip;
     }


  // INSERT>> Your destructor code here.

}

void
JPTooltipSupport::EnableHookProc(bool Enable)
{
   if (Enable)
   	{
      #if defined(BI_PLAT_WIN16)
        lpfnFilterProc = (FARPROC) ::MakeProcInstance((FARPROC) HookFilterFunc, parentwindow->GetModule()->GetInstance());
        if (lpfnFilterProc)
           lpfnOldHook = (HHOOK) ::SetWindowsHookEx(WH_GETMESSAGE, (HOOKPROC) lpfnFilterProc, (HINSTANCE) parentwindow->GetModule()->GetInstance(), GetCurrentTask());
      #elif defined(BI_PLAT_WIN32)
        lpfnOldHook    = (HHOOK) ::SetWindowsHookEx(WH_GETMESSAGE, (HOOKPROC) HookFilterFunc, (HINSTANCE) NULL, GetCurrentThreadId());
      #endif
      }
   else
   	{
      #if defined(BI_PLAT_WIN16)
        if (lpfnFilterProc)
           ::FreeProcInstance((FARPROC) lpfnFilterProc);
      #endif
        if (lpfnOldHook)
           if (::UnhookWindowsHookEx(lpfnOldHook)!=0)
           	  lpfnOldHook = NULL;
      }
}

void
JPTooltipSupport::AssociateTooltips(HWND hwnd, bool IsAssociate, bool FullTooltipSupport,
                                    bool AlwaysForceRectAssociate,
                                    bool ForceRectAssociateIfDisabled)
{
  _TCHAR szClass[64];
  _TCHAR szBuf[255];

  HWND child = ::GetWindow(hwnd, GW_CHILD);
  TToolInfo ti((HWND) NULL, (HWND) child, NULL);
  while (::IsWindow(child)) {
#if defined(BI_PLAT_WIN32)
      LONG id = ::GetWindowLong(child, GWL_ID);
#else
      WORD id = ::GetWindowWord(child, GWW_ID);
#endif
      if ((id > 0) && (id != 0xFFFF))
      	{
         int len = RetrieveTooltipString(szBuf, id);
         if (len > 0)
            {
            #if defined(BI_PLAT_WIN32)
            ti.SetText(szBuf, false);
            #elif defined(BI_PLAT_WIN16)
            ti.SetText(szBuf, true);
            #endif
            ::GetClassName(child, szClass, sizeof(szClass));
            if (((_tcscmpi(szClass, "STATIC") == 0) && FullTooltipSupport) ||
                ((!::IsWindowEnabled(child)) && ForceRectAssociateIfDisabled)
                || AlwaysForceRectAssociate)
            	{
               TRect rect;
               ::GetWindowRect(child, &rect);
               TPoint pt (rect.left, rect.top);
               ::ScreenToClient(hwnd, &pt);
               rect.right  = rect.Width() + pt.x;
               rect.bottom = rect.Height() + pt.y;
               rect.left = pt.x;
               rect.top = pt.y;
               ti.SetToolInfo((HWND) hwnd, (uint) id, rect);
               if (IsAssociate)
                  HookProcTooltip->AddTool(ti);
               else
                  HookProcTooltip->DeleteTool(ti);
               }
            else
            	{
            	ti.uId = (UINT) child;
            	ti.SetToolInfo((HWND) child, (HWND) hwnd);
               if (IsAssociate)
                  HookProcTooltip->AddTool(ti);
               else
                  HookProcTooltip->DeleteTool(ti);
               }
            }
         }
      child = ::GetWindow(child, GW_HWNDNEXT);
  }
}

int
JPTooltipSupport::RetrieveTooltipString(_TCHAR* szBuf, uint menuItemId)
{
   _TCHAR tempstr[255];
   _TCHAR* ptr = NULL, c = '\n';
   _tcscpy(tempstr,"");
   _tcscpy(szBuf,  "");
   int len = ::LoadString(parentwindow->GetModule()->GetInstance(), menuItemId, tempstr, sizeof(tempstr));
   if (len > 0)
      ptr = _tcschr(tempstr, c);
   if (ptr)
      _tcsncpy(szBuf, ptr + 1, len);
   else
      _tcscpy(szBuf, tempstr);
   return _tcslen(szBuf);
}

