//----------------------------------------------------------------------------
// ObjectWindows                                                             -
// Copyright (c) 1995, 1995 by Borland International, All Rights Reserved    -
//                                                                           -
// Illustrates usage of PropertySheets and PropertyPages                     -
//----------------------------------------------------------------------------
#include <owl/pch.h>
#if !defined(OWL_COMMCTRL_H)
# include <owl/commctrl.h>
#endif
#if !defined(OWL_CHOOSECO_H)
# include <owl/chooseco.h>
#endif
#if !defined(OWL_DECFRAME_H)
# include <owl/decframe.h>
#endif
#if !defined(OWL_STATUSBA_H)
# include <owl/statusba.h>
#endif

#include <stdio.h>

#include "jpprpage.h"
#include "jpdialog.h"
#include "propsht.h"

static JPTooltipSupport*  exttooltip;

//
// class TWinAttrDlg
// ~~~~~ ~~~~~~~~~~~
class TWinAttrDlg : public JPPropertyPage {
  public:
    TWinAttrDlg(JPPropertySheet* parent);

    // Overriden virtuals
    //
    void              SetupWindow();
    int               Apply(TNotify&);
    void              Help(TNotify&);

    // Event handlers
    //
    void              BkColorClick();
    void              EnChange();

    TColor            BkColor;
    TAPointer<char>   Caption;

  DECLARE_RESPONSE_TABLE(TWinAttrDlg);
};

TWinAttrDlg::TWinAttrDlg(JPPropertySheet* parent)
            :JPPropertyPage(parent, IDD_WINATTR, "WinAttrDlg", IDI_COLOR, 0,
                            true, false, true, true)
{
}

DEFINE_RESPONSE_TABLE1(TWinAttrDlg, JPPropertyPage)
    EV_BN_CLICKED(IDC_BKCOLOR, BkColorClick),
    EV_EN_CHANGE(IDC_TEXTEDIT, EnChange),
END_RESPONSE_TABLE;

void
TWinAttrDlg::SetupWindow()
{
  JPPropertyPage::SetupWindow();

  // Grab client window's caption
  //
  Caption = new char[_MAX_PATH];
  GetApplication()->GetMainWindow()->GetWindowText(Caption, _MAX_PATH);
  SetDlgItemText(IDC_TEXTEDIT, Caption);

  // Client's color is the default window color [??]
  //
  BkColor = TColor::SysWindow;
}

void
TWinAttrDlg::BkColorClick()
{
  TChooseColorDialog::TData colorData;
  TAPointer<TColor> custColors = new TColor[16];
  colorData.Flags = CC_RGBINIT;
  colorData.Color = BkColor;
  colorData.CustColors = custColors;
  if (TChooseColorDialog(this, colorData).Execute() == IDOK) {
    if (colorData.Color != BkColor) {
      BkColor = colorData.Color;
      GetSheet()->PageChanged(*this);
    }
  }
}

void
TWinAttrDlg::EnChange()
{
  char text[_MAX_PATH];
  text[0] = 0;
  GetDlgItemText(IDC_TEXTEDIT, text, sizeof(text));
  if (strcmp(text, Caption))
    GetSheet()->PageChanged(*this);
}

int               
TWinAttrDlg::Apply(TNotify& not)
{
  // Set color
  //
  GetApplication()->GetMainWindow()->GetClientWindow()->SetBkgndColor(BkColor);
  GetApplication()->GetMainWindow()->GetClientWindow()->Invalidate();

  // And caption
  //
  char text[_MAX_PATH];
  text[0] = 0;
  GetDlgItemText(IDC_TEXTEDIT, text, sizeof(text));
  if (strcmp(text, Caption)) {
    GetApplication()->GetMainWindow()->SetWindowText(text);
    Caption = strnewdup(text);
  }

  // These changes are not 'Cancellable'
  //
  GetSheet()->CancelToClose();

  return JPPropertyPage::Apply(not);
}

void
TWinAttrDlg::Help(TNotify& not)
{
  JPPropertyPage::Help(not);
  MessageBox("Invoke WinHelp() here", GetModule()->GetName(), MB_OK);
}

//----------------------------------------------------------------------------

//
// class TTestPage
// ~~~~~ ~~~~~~~~~
class TTestPage : public JPPropertyPage {
  public:
    TTestPage(JPPropertySheet* parent);

    // Event handlers
    //
    void EvCommandAndId(uint id);

    // Override virtuals of JPPropertyPage
    //
    int   SetActive(TNotify&);    // PSN_SETACTIVE
    bool  KillActive(TNotify&);   // PSN_KILLACTIVE
    int   Apply(TNotify&);        // PSN_APPLY
    void  Reset(TNotify&);        // PSN_RESET
    void  Help(TNotify&);         // PSN_HELP
    bool  QueryCancel(TNotify&);  // PSN_QUERYCANCEL

    DECLARE_RESPONSE_TABLE(TTestPage);
};

TTestPage::TTestPage(JPPropertySheet* parent)
          :JPPropertyPage(parent, IDD_SAMPLEPAGE, "TestPage", 0, 0, true, false, true, true)
{
}

DEFINE_RESPONSE_TABLE1(TTestPage, JPPropertyPage)

  // Handlers of control specific to this page/dialog
  //
  EV_COMMAND_AND_ID(IDC_SIM_APPLY,        EvCommandAndId),
  EV_COMMAND_AND_ID(IDC_SIM_XCELTOCLOSE,  EvCommandAndId),
  EV_COMMAND_AND_ID(IDC_SIM_CHANGED,      EvCommandAndId),
  EV_COMMAND_AND_ID(IDC_SIM_UNCHANGED,    EvCommandAndId),

  EV_COMMAND_AND_ID(IDC_ADDPAGE,          EvCommandAndId),
  EV_COMMAND_AND_ID(IDC_REMPAGE,          EvCommandAndId),
  EV_COMMAND_AND_ID(IDC_SELPAGE,          EvCommandAndId),
  EV_COMMAND_AND_ID(IDC_PAGEINFO,         EvCommandAndId),
END_RESPONSE_TABLE;

void
TTestPage::EvCommandAndId(uint id)
{
  switch(id) {
    case IDC_SIM_APPLY:
         GetSheet()->Apply();
         break;

    case IDC_SIM_XCELTOCLOSE:
         GetSheet()->CancelToClose();
         break;

    case IDC_SIM_CHANGED:
         GetSheet()->PageChanged(*this);
         break;

    case IDC_SIM_UNCHANGED:
         GetSheet()->PageUnchanged(*this);
         break;
  }
}

int   
TTestPage::SetActive(TNotify& not)    
{
  return JPPropertyPage::SetActive(not);
}

bool  
TTestPage::KillActive(TNotify& not)
{
  return JPPropertyPage::KillActive(not);
}

int   
TTestPage::Apply(TNotify& not)
{
  return JPPropertyPage::Apply(not);
}

void  
TTestPage::Reset(TNotify& not)
{
  JPPropertyPage::Reset(not);
}

void
TTestPage::Help(TNotify& not)
{
  JPPropertyPage::Help(not);
  MessageBox("Invoke WinHelp() here", GetModule()->GetName(), MB_OK);
}

bool  
TTestPage::QueryCancel(TNotify& not)
{
  return JPPropertyPage::QueryCancel(not);
}

/*******************************************************************************
*                                                                              *
*   Class TSampleAboutDlg                                                      *
*   ~~~~~ ~~~~~~~~~~~~~--                                                      *
*******************************************************************************/
class TSampleAboutDlg : public JPDialog {
  public:
    TSampleAboutDlg(TWindow* parent,
                    TResId resId,
                    TModule* module,
                    JPTooltipSupport* mytooltip,
                    bool FullTooltipSupport,
                    bool AlwaysForceRectAssociate,
                    bool ForceRectAssociateIfDisabled,
                    bool EnableStatusbarMessages);
    virtual ~TSampleAboutDlg();
};

TSampleAboutDlg::TSampleAboutDlg(TWindow* parent,
                                 TResId resId,
                                 TModule* module,
                                 JPTooltipSupport* mytooltip,
                                 bool FullTooltipSupport,
                                 bool AlwaysForceRectAssociate,
                                 bool ForceRectAssociateIfDisabled,
                                 bool EnableStatusbarMessages)
:
  JPDialog(parent, resId, module, mytooltip, FullTooltipSupport,
           AlwaysForceRectAssociate, ForceRectAssociateIfDisabled, EnableStatusbarMessages)
{
  // INSERT>> Your constructor code here.
}


TSampleAboutDlg::~TSampleAboutDlg()
{
  Destroy();
}

//----------------------------------------------------------------------------
// class TClientWindow //
// ~~~~~ ~~~~~~~~~~~~~-//
class TClientWindow : public TWindow {

  public:
    TClientWindow(TWindow* parent= 0);

  protected:
    // Message Handlers
    //
    void        Properties(bool modal);
    void        PropertiesModal()    { Properties(true);  }
    void        PropertiesModeless() { Properties(false); }
    void        AboutDialog();
  DECLARE_RESPONSE_TABLE(TClientWindow);
};

DEFINE_RESPONSE_TABLE1(TClientWindow, TWindow)
  EV_COMMAND(CM_PROPERTY, PropertiesModal),
  EV_COMMAND(CM_PROPMODELESS, PropertiesModeless),
  EV_COMMAND(CM_ABOUTDIALOG, AboutDialog),
END_RESPONSE_TABLE;


TClientWindow::TClientWindow(TWindow* parent)
:
  TWindow(parent)
{
  Attr.Style |= (WS_CLIPSIBLINGS|WS_CLIPCHILDREN);
}

void
TClientWindow::Properties(bool modal)
{
  // Allocate a PropertySheet object
  //
  JPPropertySheet* ps = new JPPropertySheet(this, "Options", 0, false,
                                PSH_DEFAULT | PSH_HASHELP, 0, exttooltip,
                                true, false, true, true);

  // Allocate PropertyPageEx objects using the sheet as 'parent' parameters
  //
  (new TWinAttrDlg(ps))->SetFlags(PSP_HASHELP);
  (new TTestPage(ps))->SetFlags(PSP_HASHELP);

  if (modal)
    ps->Execute();
  else
    ps->Create();
}

void
TClientWindow::AboutDialog()
{
  // Show the modal dialog.
  //
  TSampleAboutDlg(this, IDD_ABOUT, 0, exttooltip, true, false, true, true).Execute();
}

//----------------------------------------------------------------------------

//
// class TSampleApp
// ~~~~~ ~~~~~~~~~~
class TSampleApp : public TApplication {
  public:
    void    InitMainWindow();

  protected:
    TDecoratedFrame* frame;
};

void
TSampleApp::InitMainWindow()
{
  EnableCtl3d();
  SetName("Property Sheet");
  frame = new TDecoratedFrame(0, GetName(), new TClientWindow(), true);
  frame->AssignMenu(IDM_APPMENU);
  TStatusBar* sb = new TStatusBar(frame, TGadget::Recessed, TStatusBar::CapsLock |
                                                            TStatusBar::NumLock  |
                                                            TStatusBar::ScrollLock |
                                                            TStatusBar::SizeGrip);
  sb->SetWideHints(false);
  frame->Insert(*sb, TDecoratedFrame::Bottom);
  SetMainWindow(frame);
  exttooltip = new JPTooltipSupport(frame);
}

int
OwlMain(int /*argc*/, char*/*argv*/[])
{
  return TSampleApp().Run();
}

